;------ display shield indicator (AL)

display_shield_bar:
        cmp     al,16
        jz      .q
        mov     di,scrn+15
        add     al,al
        add     al,al
.loop:  and     byte [di], 0xfe
        add     di,16
        inc     al
        cmp     al,64
        jnz     .loop
.q:     ret

;------ scroll the sides of the screen

scroll_sides:
        std
        mov     si,edges+1006
        mov     di,edges+1022
        mov     cx,504
        rep     movsw
        xor     ax,ax
        mov     cx,8
        rep     stosw
        cld

        call    random_number
        mov     ah,al
        and     ah,0x70
        jne     .n1
        xor     ah,ah
        mov     si,directions
        add     si,ax
        lodsb
        mov     [lsd],al
.n1:    
        mov     ax,[lspic]
        mov     [edges],ah
        mov     [edges+1],al
        mov     bl,[lsd]
        or      bl,bl
        je      .left_nothing
        jg      .left_expand

.left_shrink:
        mov     byte [lsd],-1
        cmp     ax,0x8000
        je      .left_expand
        shl     ax,1
        mov     [lspic],ax
        jmp     .left_nothing

.left_expand:
        mov     byte [lsd],1
        cmp     ax,0xfffc
        je      .left_shrink
        sar     ax,1
        mov     [lspic],ax

.left_nothing:
        call    random_number
        mov     ah,al
        and     ah,0x70
        jne     .n2
        xor     ah,ah
        mov     si,directions
        add     si,ax
        lodsb
        mov     [rsd],al
.n2:    
        mov     ax,[rspic]
        mov     [edges+14],ah
        mov     [edges+15],al
        mov     bl,[rsd]
        or      bl,bl
        je      .right_nothing
        jg      .right_expand

.right_shrink:
        mov     byte [rsd],-1
        cmp     ax,0x0005
        je      .right_expand
        shr     ax,1
        dec     al
        mov     [rspic],ax
        jmp     .right_nothing

.right_expand:
        mov     byte [rsd],1
        cmp     ax,0x3ffd
        je      .right_shrink
        shl     ax,1
        add     al,3
        mov     [rspic],ax

.right_nothing:
        ret

;------ overlay sides onto main screen

overlay_sides:
        mov     di,scrn
        mov     si,edges
        mov     cx,512
        mov     dx,2
.loop:  lodsw
        or      [di],ax
        add     di,dx
        loop    .loop
        ret

;------ display sprite 8 pixels wide
;
; CL = X coordinate
; AL = Y coordinate
; SI = sprite image (first byte = height)

display_sprite_8:
        mov     di,scrn                 ; DI -> start of destination
        sub     al,SCREEN_TOP           ; AL = screen Y coodinate
        jl      .clip_top

        xor     ah,ah                   ; AX = Y coordinate
        mov     dl,al
        shl     ax,4
        add     di,ax                   ; DI -> destination line

        lodsb                           ; read height
        mov     dh,al                   ; DH = height
        add     dl,al                   ; DL = bottom Y coordinate
        sub     dl,SCREEN_HEIGHT        ; DL = # lines past screen bottom
        jg      .clip_bottom

.clipped_y:
        xor     ah,ah
        mov     al,cl                   ; AX = X coordinate
        shr     ax,3                    ; AX = byte offset
        add     di,ax                   ; DI -> destination byte

        and     cl,7                    ; CL = bit shift
        mov     bx,14                   ; BX = dist between lines - 2

.display_loop:
        xor     ah,ah
        lodsb                           ; AX = image data
        ror     ax,cl                   ; shift image data to position
        or      ax,[di]                 ; AX = image data + background
        stosw                           ; srote changed data
        add     di,bx                   ; advance to next line
        dec     dh                      ; count down lines remaining
        jne     .display_loop
        ret

.clip_top:
        neg     al                      ; AL = # lines to skip
        mov     bl,al
        lodsb                           ; AL = full height
        sub     al,bl                   ; AL = # lines remaining
        jle     .quit
        mov     dh,al                   ; DH = height to draw
        xor     bh,bh                   ; BX = # lines to skip
        add     si,bx
        jmp     .clipped_y
.quit:  ret

.clip_bottom:
        sub     dh,dl                   ; DH = # lines OK to draw
        jg      .clipped_y
        ret

;------ clear the screen

clear_screen:
        mov     di,scrn
        xor     ax,ax
        mov     cx,512
        rep     stosw
        ret

;------ initialize the screen backgrounds

init_screen:
        mov     cx,64
.loop:  push    cx
        call    do_scroll_stars
        call    scroll_sides
        pop     cx
        loop    .loop
        ret

;------ scroll the starfield

scroll_stars:
        mov     al,[game_timer]
        and     al,1
        jnz     return_ss
do_scroll_stars:
        std
        mov     si,starfield+1006
        mov     di,starfield+1022
        mov     cx,504
        rep     movsw
        xor     ax,ax
        mov     cx,8
        rep     stosw
        cld
        call    random_number
        or      al,al
        jl      return_ss
        cmp     al,125
        jg      return_ss
        mov     cl,al
        and     cl,7
        shr     al,3
        add     al,2
        xor     ah,ah
        add     di,ax
        mov     al,1
        shl     al,cl
        stosb
return_ss:
        ret

;------ copy starfield to screen

copy_stars:
        mov     di,scrn
        mov     si,starfield
        mov     cx,512
        rep     movsw
        ret

