;------ Platinum Editon for Casio Algebra FX 2.0
;
; Programmed by Patrick Davidson (pad@ocf.berkeley.edu)
;
; This software is in the public domain.  There is no warranty.

SCREEN_TOP      equ     32
SCREEN_HEIGHT   equ     64

;------ bullet data structure

b_type  equ     0
b_x     equ     1
b_w     equ     2
b_y     equ     3
b_h     equ     4
b_dmg   equ     5
b_image equ     6
b_size  equ     8

b_count equ     30

;------ enemy data structure

e_type  equ     0
e_x     equ     1
e_w     equ     2
e_y     equ     3
e_h     equ     4
e_dmg   equ     5
e_phase equ     6
e_timer equ     7
e_image equ     8
e_dx    equ     10
e_dy    equ     11
e_size  equ     12

e_count equ     30

;------ enemy bullet data structure

f_type  equ     0
f_x     equ     1
f_w     equ     2
f_y     equ     3
f_h     equ     4
f_dx    equ     5
f_image equ     6
f_size  equ     8

f_count equ     20

;------ main program (called from platform-specific initialization)

main_game:
        call    init_data

        call    text_mode
        xor     dx,dx
        call    move_cursor
        mov     si,title_message
        call    display_string

.title_loop:
        call    get_keystroke
        call    is_exit
        je      .lose
        cmp     al,'0'
        jl      .title_loop
        cmp     al,'9'
        jg      .title_loop
        sub     al,'9'
        neg     al
        cbw
        mov     cx,1000
        mul     cx
        mov     [delay_time],ax

        call    graphics_mode
        call    init_screen

.loop   inc     word [game_timer]
        call    scroll_stars
        call    copy_stars
        call    scroll_sides
        call    overlay_sides
        call    move_player
        call    show_player
        call    player_shoot
        call    player_bullets
        call    detect_enemy_hits
        call    process_enemies
        call    enemy_bullets
        mov     al,[player_shield]
        dec     al
        jl      .lose
        inc     al
        call    display_shield_bar
        call    delay
        call    display_screen
        jmp     .loop

.lose:  call    get_keystroke
        ret

;------ bounding box test of collision
;
; SI points to (X-coordinate, width, Y-coordinate, height) structure
; BH = other X-coordinate
; BL = other width
; DH = other Y-coordinate
; DL = other height
;
; modifies only AL
;
; returns carry set for collision, clear for no collision

collision_check:                ; ensure that X1 < X2 + W2
        mov     al,[si]         ; AL = X1 
        sub     al,bh           ; AL = X1 - X2
        jc      .cc1            ; X1 < X2 implies X1 < X2 + W2
        cmp     al,bl
        jnc     .q              ; exit if (X1 - X2) >= W2
                                ; X2 <= X1 implies X2 < X1 + W1

.x_check_done:                  ; now ensure that Y1 < Y2 + H2
        mov     al,[si+2]
        sub     al,dh           ; AL = Y1 - Y2
        jc      .cc2            ; Y1 < Y2 implies Y1 < Y2 + W2
        cmp     al,dl
        ret                     ; carry set if Y1 < Y2 + H2
                                ; Y2 <= Y1 implies Y2 < Y1 + H1

.cc2:   neg     al              ; ensure that Y2 < Y1 + H1
        cmp     al,[si+3]
        ret                     ; carry set if Y2 < Y1 + H1

.cc1:   neg     al              ; ensure that X2 < X1 + W1
        cmp     al,[si+1]
        jc      .x_check_done   ; continue if X2 - X1 < W1
.q:     ret


;------ very lame "random" number generation

random_number:
        mov     si,[random_pointer]
        cs lodsb
        mov     ah,al
        cs lodsb
        add     al,ah
        mov     [random_pointer],si
        sub     si,clear_screen
        jl      .done
        mov     word [random_pointer],main_game
.done:  ret

;------ initialize a new game

init_data:
        mov     di,clear_start
        mov     cx,clear_end-clear_start
        xor     ax,ax
        rep     stosb
        ret

;------ some very important files

%include "bullets.asm"
%include "player.asm"
%include "ebullets.asm"
%include "shop.asm"
%include "enemies.asm"
%include "io.asm"
%include "gfx.asm"
%include "levels.asm"

;------ miscellaneous data

directions:
        db      0,-1,1,1,-1,-1,1,0,1,-1,-1,0,0,1,-1,1

title_message:
        db      "Platinum Edition 0.10"
        db      " by Patrick Davidson",10,13
        db      "E-Mail - pad@calc.org"
        db      " http://pad.calc.org",10,13
        db      "Press 0 to 9 to pick",10,13
        db      "speed and begin game",10,13
        db      "In game, CTRL shoots",10,13
        db      "and never press ESC",0

shop_message:
        db      "Platinum Shop $0000",10,13
        db      "Use F1, up, down",10,13
        db      "> Exit the shop",10,13
        db      "  $50 Shield boost",10,13
        db      "  $200 Weapon F2",10,13
        db      "  $500 Weapon F3",0

;------ temporary variables

enemies_left:
        db      0
random_pointer:
        dw      0
lspic:  dw      0xf000
rspic:  dw      0x000d
lsd:    db      0
rsd:    db      0
px:     db      50
py:     db      88
cash:   dw      0
player_shield:
        db      16

scrn:   resb    1024
starfield:
        resb    1024
edges:  resb    1024

clear_start:

weapon_selected:
        resb    1
weapon_f2:
        resb    1
weapon_f3:
        resb    1
shop_position:
        resb    1
lose_game:
        resb    1
level_number:
        resb    1
game_timer:
        resw    1
just_shot:
        resb    1
delay_time:
        resw    1
b_array:
        resb    b_size*b_count
e_array:
        resb    e_size*e_count
f_array:
        resb    f_size*f_count
clear_end:

SEGMENT stack   STACK

        resb    400
stack_end:
